# -*- coding: utf-8 -*-

from eea.facetednavigation.interfaces import IFacetedLayout
from hashlib import md5
from ideabox.policy import utils
from plone.memoize import ram

import time


def faceted_query_handler(obj, event):
    if obj.REQUEST.ACTUAL_URL.endswith("@@faceted_query"):
        try:
            layout = IFacetedLayout(obj).layout
        except TypeError:
            layout = None
        if layout == "faceted-project":
            if "sort_on" in event.query and event.query["sort_on"] == "random_sort":
                randomizer = Randomizer(obj.REQUEST)
                event.query["sort_on"] = randomizer.random_sort_key
                event.query["sort_order"] = randomizer.random_sort_order


def _cache_key(function, generator):
    return (generator, time.time() // (12 * 60 * 60))


@ram.cache(_cache_key)
def random_key_generator(generator):
    if not isinstance(generator, bytes):
        encoded_generator = generator.encode("utf8")
    else:
        encoded_generator = generator
    return md5(encoded_generator).hexdigest()


class Randomizer(object):

    _keys = {
        0: "UID",
        1: "project_random_1",
        2: "project_random_2",
        3: "project_random_3",
        4: "project_random_4",
        5: "project_random_5",
        6: "project_random_6",
        7: "project_random_7",
    }
    _sort_order_keys = ("ascending", "descending")

    def __init__(self, request):
        self.request = request
        now = utils.now()
        generator = "{0}-{1}".format(
            self.request.cookies.get(
                "__ac", self.request.cookies.get("beaker.session")
            ),
            now.strftime("%H"),
        )
        self.key = random_key_generator(generator.encode("utf8"))

    @property
    def random_sort_key(self):
        key = ord(self.key[0])
        return self._keys[key % len(self._keys)]

    @property
    def random_sort_order(self):
        key = ord(self.key[-1])
        return self._sort_order_keys[key % len(self._sort_order_keys)]



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone.app.registry.browser import controlpanel
from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
from plone.autoform import directives as form
from z3c.form.interfaces import INPUT_MODE
from zope import schema
from zope.interface import Interface


class InvalidEmailError(schema.ValidationError):
    __doc__ = "Please enter a valid e-mail address."


class IIdeaBoxSettingsSchema(Interface):

    project_manager_email = schema.TextLine(
        title=_("Email address of the project manager"),
        description=_(
            "If there are multiple email addresses, separate them with semicolons"
        ),
    )

    form.widget("legal_information_text", klass="pat-tinymce")
    legal_information_text = schema.Text(
        title=_("Legal information text"),
        required=False,
        description=_("Legal information text"),
    )

    project_directly_submitted = schema.Bool(
        title=_("Projects directly submitted"),
        description=_("If checked, projects are public as soon as they are submitted."),
        default=True,
    )

    ts_project_submission_path = schema.TextLine(
        title=_("Path to e-guichet project form"),
        required=False,
        description=_(
            "Specify the path to the e-guichet project form. If not exist, system use default ideabox citizen form."
        ),
    )


class IdeaBoxSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IIdeaBoxSettingsSchema
    label = _("Configuration for ideabox product")
    description = _("")

    def updateFields(self):
        super(IdeaBoxSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(IdeaBoxSettingsEditForm, self).updateWidgets()
        self.fields["legal_information_text"].widgetFactory[
            INPUT_MODE
        ] = WysiwygFieldWidget


class IdeaBoxSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = IdeaBoxSettingsEditForm



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import getToolByName
from ideabox.policy import _
from ideabox.policy import utils
from ideabox.policy.content.project import IProject
from ideabox.policy.content.project import Project
from ideabox.policy.content.project import ProjectView
from plone import api
from plone.app.textfield.value import IRichTextValue
from plone.indexer.decorator import indexer
from zope import schema
from zope.interface import implementer
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class IPriorityAction(IProject):
    """IPriorityActionIProject"""

    strategic_objectives = schema.TextLine(
        title=_("Strategic Objectives"), required=False
    )

    operational_objectives = schema.TextLine(
        title=_("Operational objectives"), required=False
    )


@implementer(IPriorityAction)
class PriorityAction(Project):
    pass


class PriorityActionView(ProjectView):
    def get_state_progress(self):
        return api.content.find(
            context=self.context,
            portal_type="state_progress",
            sort_on="state_date",
            sort_order="descending",
        )[:4]

    def format_date(self, value):
        return utils.localized_month(value.strftime("%d %B %Y"), self.request)

    def get_restapi_viewlets(self):
        manager = queryMultiAdapter(
            (self.context, self.request, self),
            IViewletManager,
            "plone.belowcontenttitle",
            default=None,
        )
        manager.update()
        viewlets = [v for v in manager.viewlets if v.__name__ == "imio-restapi-actions"]
        return viewlets


@indexer(IPriorityAction)
def searchabletext_priority_action(object, **kw):
    result = []

    fields = ["title", "description", "body", "original_author"]
    for field_name in fields:
        value = getattr(object, field_name, None)
        if IRichTextValue.providedBy(value):
            transforms = getToolByName(object, "portal_transforms")
            text = (
                transforms.convertTo("text/plain", value.raw, mimetype=value.mimeType)
                .getData()
                .strip()
            )
            result.append(text)
        else:
            text = value
            if text:
                result.append(text)
    return " ".join(result)



# -*- coding: utf-8 -*-

from cioppino.twothumbs.browser.like import LikeWidgetView
from plone import api
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ProjectRatingView(LikeWidgetView):
    index = ViewPageTemplateFile("templates/rating.pt")
    _allowed_states = ("vote",)

    @property
    def is_available(self):
        current_state = api.content.get_state(obj=self.context)
        return current_state in self._allowed_states

    def canRate(self):
        if self.is_available is False:
            return False
        return super(ProjectRatingView, self).canRate()



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plone.app.discussion.browser.comments import CommentForm as BaseForm
from plone.app.discussion.browser.comments import CommentsViewlet as BaseViewlet
from Products.CMFCore.utils import getToolByName

import six


class CommentForm(BaseForm):
    def get_author(self, data):
        context = aq_inner(self.context)
        # some attributes are not always set
        author_name = ""

        # Make sure author_name/ author_email is properly encoded
        if "author_name" in data:
            author_name = data["author_name"]
            if isinstance(author_name, str):
                author_name = six.text_type(author_name, "utf-8")
        if "author_email" in data:
            author_email = data["author_email"]
            if isinstance(author_email, str):
                author_email = six.text_type(author_email, "utf-8")

        # Set comment author properties for anonymous users or members
        portal_membership = getToolByName(context, "portal_membership")
        anon = portal_membership.isAnonymousUser()
        if not anon and getSecurityManager().checkPermission("Reply to item", context):
            # Member
            member = portal_membership.getAuthenticatedMember()
            # memberdata is stored as utf-8 encoded strings
            email = member.getProperty("email")
            infos = [member.getProperty("first_name"), member.getProperty("last_name")]
            fullname = " ".join([i for i in infos if i])
            if not fullname or fullname == "":
                fullname = member.getUserName()
            elif isinstance(fullname, six.text_type):
                fullname = fullname.encode("utf-8")
            author_name = fullname
            if email and isinstance(email, six.text_type):
                email = email.encode("utf-8")
            # XXX: according to IComment interface author_email must not be  # noqa T000
            # set for logged in users, cite:
            # 'for anonymous comments only, set to None for logged in comments'
            author_email = email
            # /XXX  # noqa T000

        return author_name, author_email


class CommentsViewlet(BaseViewlet):
    form = CommentForm

    def get_commenter_home_url(self, username=None):
        return



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone import schema
from plone.app.users.browser.register import BaseRegistrationForm
from plone.app.users.browser.userdatapanel import UserDataPanel
from plone.supermodel import model
from plone.z3cform.fieldsets import extensible
from z3c.form import field
from z3c.form.browser.radio import RadioFieldWidget
from zope.component import adapts
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IEnhancedUserDataSchema(model.Schema):

    last_name = schema.TextLine(title=_("Last name or institution"), required=True)

    first_name = schema.TextLine(title=_("First name"), required=False)

    address = schema.Text(title=_("Address"), required=False)

    gender = schema.Choice(
        title=_("Gender"), required=True, vocabulary="ideabox.vocabularies.gender"
    )

    birthdate = schema.Date(title=_("Birthdate"), required=True)

    zip_code = schema.Choice(
        title=_("Zip code / locality"),
        required=True,
        vocabulary="collective.taxonomy.locality",
        description=_("zip code, locality or zip code and locality"),
    )

    iam = schema.Choice(
        title=_("I am"), required=True, vocabulary="collective.taxonomy.iam"
    )


class UserDataPanelExtender(extensible.FormExtender):
    adapts(Interface, IDefaultBrowserLayer, UserDataPanel)

    def update(self):
        fields = field.Fields(IEnhancedUserDataSchema)
        fields = fields.omit("accept")
        fields["gender"].widgetFactory = RadioFieldWidget
        self.add(fields)


class RegistrationPanelExtender(extensible.FormExtender):
    adapts(Interface, IDefaultBrowserLayer, BaseRegistrationForm)

    def update(self):
        fields = field.Fields(IEnhancedUserDataSchema)
        fields["gender"].widgetFactory = RadioFieldWidget
        self.add(fields)



# -*- coding: utf-8 -*-

from plone import api
from plone.app.layout.viewlets.common import FooterViewlet


class FooterViewlet(FooterViewlet):
    """ """

    def is_gdpr(self):
        return api.portal.get_registry_record(
            "imio.gdpr.interfaces.IGDPRSettings.is_text_ready", default=False
        )



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class ICampaign(model.Schema):
    project_submission = schema.Bool(
        title=_("Enable / Disable project submission"), default=False
    )

    emails = schema.TextLine(
        title=_("Email addresses"),
        description=_(
            'Used to send notification when a new project is proposed by an user. Accept many email addresses separated with ";"'
        ),
        required=False,
    )

    ts_project_submission_path = schema.URI(
        title=_("Path to e-guichet project form"),
        required=False,
        description=_(
            "Specify the path to the e-guichet project form. If not exist, system use default ideabox citizen form."
        ),
    )


@implementer(ICampaign)
class Campaign(Container):
    pass



# -*- coding: utf-8 -*-

from ideabox.policy import _
from ideabox.policy.utils import get_district
from ideabox.policy.utils import get_theme
from operator import itemgetter
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFPlone import PloneMessageFactory as PMF
from zope.component import getUtility
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def dict_list_2_vocabulary(dict_list):
    """dict_list_2_vocabulary
    Converts a dictionary list to a SimpleVocabulary

    :param dict_list: dictionary list
    """
    terms = []
    for item in dict_list:
        for key in sorted([k for k in item]):
            terms.append(SimpleVocabulary.createTerm(key, str(key), item[key]))
    return SimpleVocabulary(terms)


class ReviewStateVocabularyFactory(object):
    def __call__(self, context=None):
        values = [
            {"draft": PMF("draft")},
            {"deposited": PMF("deposited")},
            {"project_analysis": PMF("project_analysis")},
            {"vote": PMF("vote")},
            {"result_analysis": PMF("result_analysis")},
            {"selected": PMF("selected")},
            {"rejected": PMF("rejected")},
            {"study_in_progress": PMF("study_in_progress")},
            {"in_progress": PMF("in_progress")},
            {"realized": PMF("realized")},
        ]
        return dict_list_2_vocabulary(values)


ReviewStateVocabulary = ReviewStateVocabularyFactory()


class GenderVocabularyFactory(object):
    def __call__(self, context=None):
        values = [{"MALE": _("MALE", "Male")}, {"FEMALE": _("FEMALE", "Female")}]
        return dict_list_2_vocabulary(values)


GenderVocabulary = GenderVocabularyFactory()


class ZipCodeVocabularyFactory(object):
    def __call__(self, context=None):
        registry = getUtility(IRegistry)
        dict_value = registry.get("ideabox.vocabulary.zip_code")
        values = []
        for key in dict_value:
            val = {key: dict_value[key]}
            values.append(val)
        return dict_list_2_vocabulary(values)


ZipCodeVocabulary = ZipCodeVocabularyFactory()


def make_terms(items):
    """Create zope.schema terms for vocab from tuples"""
    terms = [SimpleTerm(value=pair[0], token=pair[0], title=pair[1]) for pair in items]
    return terms


class ProjectsVocabularyFactory(object):
    def __call__(self, context=None):
        brains = api.content.find(portal_type="Project", review_state="vote")
        results = [(b.UID, b.Title) for b in brains]
        results = sorted(results, key=itemgetter(1))
        terms = make_terms(results)
        return SimpleVocabulary(terms)


ProjectsVocabulary = ProjectsVocabularyFactory()


class VoteVocabularyFactory(object):
    def __call__(self, context=None):
        values = [{"FOR": _("FOR", "For")}, {"AGAINST": _("AGAINST", "Against")}]
        return dict_list_2_vocabulary(values)


VoteVocabulary = VoteVocabularyFactory()


class SortProjectVocabularyFactory(object):
    def __call__(self, context=None):
        values = [{"random_sort": _("Random sort")}, {"sortable_title": _("Title")}]
        return dict_list_2_vocabulary(values)


SortProjectVocabulary = SortProjectVocabularyFactory()


class DistrictVocabularyFactory:
    def __call__(self, context=None):
        district_taxo = get_district()
        language = api.portal.get_current_language(context=context)
        district_voca = district_taxo.makeVocabulary(language).inv_data
        items = [
            (k, district_taxo.translate(k, target_language=language))
            for k, v in district_voca.items()
        ]
        terms = make_terms(items)
        return SimpleVocabulary(terms)


DistrictVocabulary = DistrictVocabularyFactory()


class ThemeVocabularyFactory:
    def __call__(self, context=None):
        theme_taxo = get_theme()
        language = api.portal.get_current_language(context=context)
        theme_voca = theme_taxo.makeVocabulary(language).inv_data
        items = [
            (k, theme_taxo.translate(k, target_language=language))
            for k, v in theme_voca.items()
        ]
        terms = make_terms(items)
        return SimpleVocabulary(terms)


ThemeVocabulary = ThemeVocabularyFactory()



# -*- coding: utf-8 -*-
from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from ideabox.policy import _
from ideabox.policy.form.project_submission import IProjectSubmission
from ideabox.policy.userdataschema import IEnhancedUserDataSchema
from ideabox.policy.utils import execute_under_admin
from plone import api
from plone import schema
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.registry.interfaces import IRegistry
from z3c.form import button
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import IFieldsForm
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer

import hashlib
import password_generator
import time


class IProjectEncoding(IEnhancedUserDataSchema):
    mail = schema.Email(title=_("Email"), required=False)


@implementer(IFieldsForm)
class ProjectEncodingForm(Form):
    label = _("Project encoding")
    fields = Fields(IProjectEncoding).select(
        "last_name",
        "first_name",
        "gender",
        "mail",
        "birthdate",
        "address",
        "zip_code",
        "iam",
    )

    fields = fields + Fields(IProjectSubmission).select(
        "title",
        "project_theme",
        "project_district",
        "body",
        "project_image",
        "geolocation",
    )

    fields["project_theme"].widgetFactory = MultiSelect2FieldWidget
    fields["project_district"].widgetFactory = MultiSelect2FieldWidget
    fields["gender"].widgetFactory = RadioFieldWidget

    _required_fields = ("title", "project_theme", "project_district", "body")

    ignoreContext = True

    def update(self):
        for name, field in self.fields.items():
            field.field.required = name in self._required_fields
        super(ProjectEncodingForm, self).update()

    def create_user(self, data):
        if data["mail"]:
            existing_user = api.user.get(userid=data["mail"].lower())
            if existing_user:
                self.update_user(existing_user, data)
                return existing_user
        properties = dict(
            last_name=data["last_name"]
            or translate(_("Anonymous"), context=self.request),
            first_name=data["first_name"],
            gender=data["gender"],
            birthdate=data["birthdate"],
            zip_code=data["zip_code"],
            iam=data["iam"],
            address=data["address"],
        )
        if not data["mail"]:
            normalizer = getUtility(IIDNormalizer)
            mail = "{0}_{1}_{2}@liege2025.be".format(
                normalizer.normalize(data["title"])[:20],
                normalizer.normalize(data["last_name"])[:5],
                hashlib.md5(str(time.time())).hexdigest()[:4],
            )
        else:
            mail = data["mail"]

        return api.user.create(
            email=mail.lower(),
            password="@1aA{0}".format(password_generator.generate()),
            properties=properties,
        )

    def update_user(self, user, data):
        updates = {}
        new_properties = {
            "last_name": data["last_name"]
            or translate(_("Anonymous"), context=self.request),
            "first_name": data["first_name"],
            "gender": data["gender"],
            "birthdate": data["birthdate"],
            "zip_code": data["zip_code"],
            "iam": data["iam"],
            "address": data["address"],
        }
        for field, new_value in new_properties.items():
            if not new_value:
                continue
            if user.getProperty(field) != new_value:
                updates[field] = new_value
        user.setMemberProperties(mapping=updates)

    def send_request(self, data):
        user = self.create_user(data)

        registry = getUtility(IRegistry)
        folder = registry.get("ideabox.new.project.folder")
        if not folder.startswith("/"):
            folder = "/{0}".format(folder)
        container = api.content.get(path=folder)
        project_obj = execute_under_admin(
            container,
            api.content.create,
            type="Project",
            title=data["title"],
            project_theme=data["project_theme"],
            project_district=data["project_district"],
            body=data["body"],
            container=container,
            original_author=user.id,
        )
        execute_under_admin(
            container, api.content.transition, obj=project_obj, transition="deposit"
        )
        if data["project_image"]:
            execute_under_admin(
                project_obj,
                api.content.create,
                type="Image",
                title=data["project_image"].filename,
                image=data["project_image"],
                container=project_obj,
            )
        self.request.response.redirect(project_obj.absolute_url())

    @button.buttonAndHandler(_("Send"), name="send")
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.send_request(data)



# -*- coding: utf-8 -*-

from collective.faceted.map import _
from collective.faceted.map.browser.map import MapView
from ideabox.policy.utils import can_view_rating
from plone import api
from urllib.parse import urlparse
from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory

import json
import requests


class ProjectsView(MapView):
    @property
    def map_configuration(self):
        map_layers = api.portal.get_registry_record("geolocation.map_layers") or []
        config = {
            "fullscreencontrol": api.portal.get_registry_record(
                "geolocation.fullscreen_control", default=True
            ),
            "locatecontrol": api.portal.get_registry_record(
                "geolocation.locate_control", default=True
            ),
            "zoomcontrol": api.portal.get_registry_record(
                "geolocation.zoom_control", default=True
            ),
            "minimap": api.portal.get_registry_record(
                "geolocation.show_minimap", default=True
            ),
            "addmarker": api.portal.get_registry_record(
                "geolocation.show_add_marker", default=False
            ),
            "geosearch": api.portal.get_registry_record(
                "geolocation.show_geosearch", default=False
            ),
            "geosearch_provider": api.portal.get_registry_record(
                "geolocation.geosearch_provider", default=[]
            ),
            "default_map_layer": api.portal.get_registry_record(
                "geolocation.default_map_layer", default=[]
            ),
            "latitude": api.portal.get_registry_record(
                "geolocation.default_latitude", default=0.0
            ),
            "longitude": api.portal.get_registry_record(
                "geolocation.default_longitude", default=0.0
            ),
            "map_layers": [
                {
                    "title": translate(_(map_layer), context=self.request),
                    "id": map_layer,
                }
                for map_layer in map_layers
            ],
            "useCluster": False,
        }
        return json.dumps(config)

    def default_image(self, obj=None):
        if getattr(obj, "project_image", None) is None:
            return "{0}/project_default.jpg".format(api.portal.get().absolute_url())
        return "{}/@@images/project_image/project_faceted".format(obj.absolute_url())

    def rating(self, context):
        return can_view_rating(context)

    def get_theme(self, key):
        if not hasattr(self, "_themes"):
            factory = getUtility(IVocabularyFactory, "collective.taxonomy.theme")
            self._themes = factory(self.context)
        try:
            return self._themes.getTerm(key).title
        except KeyError:
            return ""

    def get_path(self):
        context = self.context
        return "/".join(context.getPhysicalPath())

    def can_submit_project(self):
        if self.context.portal_type != "campaign":
            return False
        return getattr(self.context, "project_submission", False)

    @property
    def is_there_eguichet_project_form(self):
        if api.portal.get_registry_record(
            "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.ts_project_submission_path"
        ):
            return True
        return False

    @property
    def open_project_form(self):
        form_url = ""
        campagne_ideabox_path = urlparse(self.context.absolute_url()).path
        if getattr(self.context, "ts_project_submission_path", None) is not None:
            form_url = "window.open('{}/?campagne={}','_blank')".format(
                self.context.ts_project_submission_path, campagne_ideabox_path
            )
        elif self.is_there_eguichet_project_form:
            eguichet_form_path = api.portal.get_registry_record(
                "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.ts_project_submission_path"
            )
            form_url = "window.open('{}/?campagne={}','_blank')".format(
                eguichet_form_path, campagne_ideabox_path
            )
        else:
            form_url = "window.open('{}/{}','_self')".format(
                self.get_path(), "@@project_submission"
            )
        return form_url



from ideabox.policy.testing import IdeaboxTestCase
from ideabox.policy.testing import IDEABOX_POLICY_INTEGRATION_TESTING
from plone import api
from zope.component import getSiteManager
from collective.taxonomy.interfaces import ITaxonomy
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID


class TestVocabularies(IdeaboxTestCase):

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_vote(self):
        self.assertVocabularyLen("ideabox.vocabularies.vote", 2)

    def test_review_state(self):
        self.assertVocabularyLen("ideabox.vocabularies.review_states", 10)

    def test_gender(self):
        self.assertVocabularyLen("ideabox.vocabularies.gender", 2)

    def test_zip_code(self):
        zip_code = api.portal.get_registry_record("ideabox.vocabulary.zip_code")
        self.assertVocabularyLen("ideabox.vocabularies.zip_code", len(zip_code))

    def test_projects(self):
        self.assertVocabularyLen("ideabox.vocabularies.projects", 0)
        campaign = api.content.create(
            container=self.portal, type="campaign", title="Campaign1"
        )
        project = api.content.create(
            container=campaign,
            type="Project",
            title="Project1",
        )
        self.assertVocabularyLen("ideabox.vocabularies.projects", 0)
        api.content.transition(obj=project, to_state="vote")
        self.assertVocabularyLen("ideabox.vocabularies.projects", 1)

    def test_sort_project(self):
        self.assertVocabularyLen("ideabox.vocabularies.sort_project", 2)

    def test_district(self):
        self.assertVocabularyLen("ideabox.vocabularies.district", 4)

    def test_theme(self):
        sm = getSiteManager()
        themes = sm.queryUtility(ITaxonomy, name="collective.taxonomy.theme")
        theme_voca = themes.makeVocabulary("fr")
        self.assertVocabularyLen("ideabox.vocabularies.theme", len(theme_voca))



# -*- coding: utf-8 -*-

from plone.api import portal as portal_api
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from unittest.mock import Mock
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


import ideabox.policy
import unittest


class IdeaboxPolicyLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(name="testing.zcml", package=ideabox.policy)

    def setUpPloneSite(self, portal):
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")
        portal_api.get_current_language = Mock(return_value="fr")
        applyProfile(portal, "ideabox.policy:default")


IDEABOX_POLICY_FIXTURE = IdeaboxPolicyLayer()


IDEABOX_POLICY_INTEGRATION_TESTING = IntegrationTesting(
    bases=(IDEABOX_POLICY_FIXTURE,), name="IdeaboxPolicyLayer:IntegrationTesting"
)


IDEABOX_POLICY_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(IDEABOX_POLICY_FIXTURE,), name="IdeaboxPolicyLayer:FunctionalTesting"
)


IDEABOX_POLICY_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(IDEABOX_POLICY_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="IdeaboxPolicyLayer:AcceptanceTesting",
)


class IdeaboxTestCase(unittest.TestCase):
    def assertVocabularyLen(self, vocname, voc_len):
        factory = getUtility(IVocabularyFactory, vocname)
        vocabulary = factory()
        self.assertEqual(len(vocabulary), voc_len)



# -*- coding: utf-8 -*-
from ideabox.policy import _
from ideabox.policy import utils
from plone import api
from plone.app.textfield import RichText
from plone.dexterity.content import Container
from plone.supermodel import model
from Products.Five import BrowserView
from zope import schema
from zope.interface import implementer


class IStateProgress(model.Schema):
    """IStateProgress"""

    state_date = schema.Date(title=_("Date"), required=True)

    body = RichText(title=_("Content"), required=True)


@implementer(IStateProgress)
class StateProgress(Container):
    pass


class StateProgressView(BrowserView):
    def get_state_progress(self):
        return api.content.find(
            context=self.context,
            portal_type="state_progress",
            sort_on="state_date",
            sort_order="descending",
        )

    def format_date(self, value):
        return utils.localized_month(value.strftime("%d %B %Y"), self.request)



# encoding: utf-8

from hashlib import md5
from ideabox.policy import _
from ideabox.policy.faceted.subscriber import Randomizer
from ideabox.policy.utils import can_view_rating
from plone import api
from plone.supermodel.model import Schema
from plone.tiles import Tile
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from random import shuffle
from zope import schema
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import time


class IProjectsTile(Schema):
    """A tile that displays a listing of content items"""

    title = schema.TextLine(title=_("Title"), required=True, default=_("Projects"))

    limit = schema.Int(
        title=_("Limit"),
        description=_("Limit projects"),
        required=False,
        default=6,
        min=1,
        max=15,
    )


class TileRandomizer(Randomizer):
    def __init__(self):
        self.key = md5(str(time.time() // 60 * 5).encode("utf-8")).hexdigest()


class BaseProjectsTile(Tile):
    template = ViewPageTemplateFile("templates/projects.pt")

    def __call__(self):
        return self.template()

    @property
    def title(self):
        return self.data.get("title") or _("Projects")

    def contents(self):
        limit = self.data["limit"]
        return api.content.find(
            portal_type="Project", sort_on="created", sort_order="reverse"
        )[:limit]

    def folder_projects(self):
        folder = api.content.find(portal_type="Folder", id="projets")
        if len(folder) == 1:
            return folder[0].getURL()
        return False

    def get_theme(self, key):
        if not hasattr(self, "_themes"):
            factory = getUtility(IVocabularyFactory, "collective.taxonomy.theme")
            self._themes = factory(self.context)
        try:
            return self._themes.getTerm(key).title
        except KeyError:
            return ""

    @property
    def default_image(self):
        return "{0}/project_default.jpg".format(api.portal.get().absolute_url())

    def rating(self, context):
        return can_view_rating(context)


class LatestProjectsTile(BaseProjectsTile):
    """A tile that displays a listing of content items"""

    def contents(self):
        limit = self.data["limit"]
        return api.content.find(
            portal_type="Project", sort_on="created", sort_order="reverse"
        )[:limit]


class RandomProjectsTile(BaseProjectsTile):
    """A tile that displays random projects"""

    def contents(self):
        limit = self.data["limit"]
        query = {}
        randomizer = TileRandomizer()
        query["sort_on"] = randomizer.random_sort_key
        query["sort_order"] = randomizer.random_sort_order
        result = api.content.find(portal_type="Project", **query)[:200]
        shuffle(result)
        return result[:limit]



# -*- coding: utf-8 -*-

from ideabox.policy import _
from plone import api
from Products.Five import BrowserView


class SummaryProgressionView(BrowserView):
    @property
    def title(self):
        return _("Projects in progression")

    @property
    def filter_widget(self):
        return "c6"

    def get_summary(self):
        return (
            {
                "title": "study_in_progress",
                "count": self.count("study_in_progress"),
                "description": _("diagnose and analysis"),
            },
            {
                "title": "in_progress",
                "count": self.count("in_progress"),
                "description": _("Setting up"),
            },
            {"title": "realized", "count": self.count("realized"), "description": None},
        )

    def count(self, state):
        return len(
            api.content.find(
                context=self.context, portal_type="Project", review_state=state
            )
        )


class SummaryThemeView(BrowserView):
    @property
    def title(self):
        return _("The 7 main themes")

    @property
    def filter_widget(self):
        return "c1"

    def get_summary(self):
        return [
            {
                "title": t.title,
                "count": self.count(t.remoteUrl[-4:]),
                "description": None,
                "code": t.remoteUrl[-4:],
            }
            for t in api.portal.get()["projets"]["par-theme"].listFolderContents()
        ]

    def count(self, theme):
        return len(
            api.content.find(
                context=self.context.get("projets", self.context),
                portal_type="Project",
                project_theme=theme,
            )
        )



from ideabox.policy.testing import IdeaboxTestCase
from ideabox.policy.testing import IDEABOX_POLICY_INTEGRATION_TESTING
from plone import api
from zope.component import getSiteManager
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from plone.i18n.interfaces import ILanguageSchema
from collective.taxonomy.interfaces import ITaxonomy
from ideabox.policy.utils import can_view_rating
from ideabox.policy.utils import review_state
from ideabox.policy.utils import localized_month
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.i18n.utility import setLanguageBinding


class TestUtils(IdeaboxTestCase):

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_review_state(self):
        campaign = api.content.create(
            container=self.portal,
            type="campaign",
            title="Campaign1",
        )
        project = api.content.create(
            container=campaign, type="Project", title="Project1"
        )
        api.content.transition(obj=project, to_state="vote")
        self.assertEqual(review_state(project), "vote")

    def test_localized_month(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILanguageSchema, prefix="plone")
        settings.use_request_negotiation = True
        settings.available_languages.append("fr")
        self.request.set("HTTP_ACCEPT_LANGUAGE", "fr")
        setLanguageBinding(self.request)
        self.assertEqual(
            localized_month("November 2025", self.request), "Novembre 2025"
        )
        self.assertEqual(localized_month("Kamoulox", self.request), "Kamoulox")

    def test_can_view_rating(self):
        campaign = api.content.create(
            container=self.portal,
            type="campaign",
            title="Campaign1",
        )
        project = api.content.create(
            container=campaign, type="Project", title="Project1"
        )
        api.content.transition(obj=project, to_state="project_analysis")
        self.assertFalse(can_view_rating(project))
        api.content.transition(obj=project, to_state="vote")
        self.assertTrue(can_view_rating(project))



# -*- coding: utf-8 -*-

from datetime import datetime
from DateTime import DateTime
from ideabox.policy import _
from plone import api
from plone.z3cform.layout import FormWrapper
from z3c.form import button
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import NO_VALUE
from ZODB.PersistentMapping import PersistentMapping
from zope import schema
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.interface.interface import InterfaceClass


class IWorkflowForm(Interface):
    """Marker interface for workflow history form"""


class WorkflowDataProvider(object):
    def get(self):
        return self.values.get(self.field.__name__, NO_VALUE)

    def __init__(self, context, request, form, field, widget):
        self.context = context
        self.request = request
        self.form = form
        self.field = field
        self.widget = widget

    @property
    def values(self):
        return {
            l.get("review_state"): l.get("time").asdatetime()
            for l in self.form.context.workflow_history.values()[0]
        }


@implementer(IWorkflowForm)
class WorkflowHistoryForm(Form):
    ignoreContext = True

    _states = (
        "deposited",
        "project_analysis",
        "vote",
        "result_analysis",
        "selected",
        "rejected",
        "study_in_progress",
        "in_progress",
        "realized",
    )

    @property
    def workflow(self):
        return self.context.workflow_history.values()[0]

    def update(self):
        current_state = api.content.get_state(obj=self.context)
        states = self._states[: self._states.index(current_state) + 1]
        for idx, state in enumerate(states):
            self.fields += Fields(
                InterfaceClass(
                    "IWorkflowHistory{0}",
                    attrs={
                        state: schema.Date(
                            title=translate(state, domain="plone"), required=True
                        )
                        for l in self.workflow
                        if l.get("review_state") == state
                    },
                )
            )
        super(WorkflowHistoryForm, self).update()

    @button.buttonAndHandler(_("Save"), name="save")
    def handleApply(self, action):
        data, errors = self.extractData()
        workflow_history = self.context.workflow_history
        for line in workflow_history.values()[0]:
            state = line.get("review_state")
            if state in data:
                line["time"] = DateTime(
                    datetime.combine(data[state], datetime.min.time())
                )
        self.context.workflow_history = PersistentMapping(workflow_history.items())
        self.request.response.redirect(self.context.absolute_url())


class WorkflowEditView(FormWrapper):
    form = WorkflowHistoryForm



# -*- coding: utf-8 -*-

from ideabox.policy.utils import get_vocabulary
from plone.restapi.services import Service


class ThemeProjectGet(Service):
    def reply(self):
        voc_theme = get_vocabulary("ideabox.vocabularies.theme")
        result = [{"id": voc.value, "text": voc.title} for voc in voc_theme]
        return result


class DistrictProjectGet(Service):
    def reply(self):
        voc_district = get_vocabulary("ideabox.vocabularies.district")
        result = [{"id": voc.value, "text": voc.title} for voc in voc_district]
        return result



# -*- coding: utf-8 -*-

from collective.geolocationbehavior.geolocation import IGeolocatable
from ideabox.policy.content.project import IProject
from ideabox.policy.testing import IdeaboxTestCase
from ideabox.policy.testing import IDEABOX_POLICY_FUNCTIONAL_TESTING
from ideabox.policy.tests.utils import make_named_image
from plone import api
from plone.app.discussion.interfaces import IConversation
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.namedfile.file import NamedBlobImage
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.component import queryUtility

# from plone.app.discussion.browser.comments import CommentForm
from ideabox.policy.browser.comment import CommentForm
from z3c.form.interfaces import IFormLayer
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component import provideAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserRequest


import transaction


class TestProject(IdeaboxTestCase):

    layer = IDEABOX_POLICY_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        self.campaign = api.content.create(
            container=self.portal,
            type="campaign",
            id="campaign",
        )

    def test_project_schema(self):
        fti = queryUtility(IDexterityFTI, name="Project")
        schema = fti.lookupSchema()
        self.assertEqual(IProject, schema)

    def test_project_fti(self):
        fti = queryUtility(IDexterityFTI, name="Project")
        self.assertTrue(fti)

    def test_project_factory(self):
        fti = queryUtility(IDexterityFTI, name="Project")
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(
            IProject.providedBy(obj),
            "IProject not provided by {0}!".format(
                obj,
            ),
        )

    def test_project_adding(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
        )

        self.assertTrue(
            IProject.providedBy(project),
            "IProject not provided by {0}!".format(
                project.id,
            ),
        )

        parent = project.__parent__
        self.assertIn("project", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=project)
        self.assertNotIn("project", parent.objectIds())

    def test_project_globally_addable(self):
        fti = queryUtility(IDexterityFTI, name="Project")
        self.assertFalse(
            fti.global_allow, "{0} is not globally addable!".format(fti.id)
        )

    def test_project_image_rendering(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        # created a project and test if title is in rendering
        self.assertIn("My new project", view())
        self.assertNotIn("project_image/large", view())

        good_default_img_id = "project_default_large.jpg"
        default_project_img = api.content.create(
            container=self.portal,
            type="Image",
            id=good_default_img_id,
            title="project_default_large.jpg",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        # created a default img with a specific id in portal and test if this default image is in project rendering
        self.assertIn("project_default_large.jpg", view())

        img = NamedBlobImage(**make_named_image())
        project.project_image = img
        view = queryMultiAdapter((project, self.request), name="view")
        # created a project_image and test if this image is in project rendering
        # this project_image take priority over default project img
        self.assertIn("project_image/large", view())
        self.assertNotIn("project_default_large.jpg", view())

        img1 = api.content.create(
            container=project, type="Image", id="img1", title="Image 1"
        )
        img2 = api.content.create(
            container=project, type="Image", id="img2", title="Image 2"
        )
        view = queryMultiAdapter((project, self.request), name="view")
        # created Images in project container
        # these images take priority over project_image
        self.assertNotIn("project_image/large", view())
        self.assertIn("img1", view())
        self.assertIn("img2", view())

    def test_project_disctrict_rendering(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class="district"', view())

        project.project_district = ["DISTRICT1", "DISTRICT2"]
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertIn('class="district"', view())
        self.assertIn("Quartier 1, Quartier 2", view())

    def test_project_theme_rendering(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class="theme"', view())

        project.project_theme = ["PROP", "CAVI"]
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertIn('class="theme"', view())
        self.assertIn(
            "<a href=http://nohost/plone/projets#b_start=0&c5=PROP>Propreté</a>, <a href=http://nohost/plone/projets#b_start=0&c5=CAVI>Cadre de vie et urbanisme</a>",
            view(),
        )

    def test_project_image_from_campaign(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class ="image campaign_image"', view())

        img = NamedBlobImage(**make_named_image())
        self.campaign.image = img
        transaction.commit()
        self.assertIn('class="image campaign_image"', view())
        self.assertIn("http://nohost/plone/campaign/@@images/image", view())

    def test_project_geolocated(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class="latitude"', view())

        IGeolocatable(project).geolocation = Geolocation(latitude=0, longitude=0)
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class="geolocation_wrapper"', view())

        IGeolocatable(project).geolocation = Geolocation(latitude="4.5", longitude="45")
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertIn('class="geolocation_wrapper"', view())
        self.assertIn('class="latitude"', view())

    def test_project_author(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My new project",
        )
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('class ="documentByLine"', view())
        self.assertNotIn('class="documentAuthor"', view())

        api.content.transition(project, "deposit")
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertIn('class="documentByLine"', view())
        self.assertNotIn('class="documentAuthor"', view())
        self.assertNotIn('<span class="authorname">', view())

        api.user.get(userid=TEST_USER_ID).setProperties(
            {"first_name": "KAMOU", "last_name": "LOX"}
        )

        view = queryMultiAdapter((project, self.request), name="view")
        self.assertIn('class="documentAuthor"', view())
        self.assertIn('<span class="authorname">KAMOU LOX', view())

        project.original_author = "Imio Test"
        view = queryMultiAdapter((project, self.request), name="view")
        self.assertNotIn('<span class="authorname">KAMOU LOX', view())
        self.assertIn('<span class="authorname">Imio Test', view())

    # def test_project_comments(self):
    #
    #     # def make_request(form={}):
    #     #     request = TestRequest()
    #     #     request.form.update(form)
    #     #     alsoProvides(request, IFormLayer)
    #     #     alsoProvides(request, IAttributeAnnotatable)
    #     #     return request
    #     #
    #     # provideAdapter(
    #     #     adapts=(Interface, IBrowserRequest),
    #     #     provides=Interface,
    #     #     factory=CommentForm,
    #     #     name="comment-form",
    #     # )
    #     #
    #     # # The form should return an error if the comment text field is empty
    #     # request = make_request(form={})
    #
    #     project = api.content.create(
    #         container=self.campaign,
    #         type="Project",
    #         id="project",
    #         title="My new project",
    #     )
    #     conversation = IConversation(project)
    #     comment1 = createObject("plone.Comment")
    #     comment1.author_name = "Kamou Lox"
    #     conversation.addComment(comment1)
    #     import transaction
    #     transaction.commit()
    #     # # commentForm = getMultiAdapter(
    #     # #     (project, request),
    #     # #     name="comment-form",
    #     # # )
    #     # # commentForm.update()
    #     # data, errors = commentForm.extractData()
    #     # import pdb;
    #     # pdb.set_trace()
    #     # commentForm.get_author(data)
    #     # import pdb;pdb.set_trace()
    #     alsoProvides(self.request, IFormLayer)
    #     view = queryMultiAdapter((project, self.request), name="view")
    #     print(view())
    #     self.assertIn("Kamou Lox", view())



# -*- coding: utf-8 -*-

from ideabox.policy.testing import IdeaboxTestCase
from ideabox.policy.testing import IDEABOX_POLICY_INTEGRATION_TESTING
from ideabox.policy.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from zope.component import queryMultiAdapter


class TestFacetedProject(IdeaboxTestCase):

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        self.campaign = api.content.create(
            container=self.portal,
            type="campaign",
            id="campaign",
        )

    def test_faceted_rendering(self):
        project = api.content.create(
            container=self.campaign,
            type="Project",
            id="project",
            title="My first project",
        )
        api.content.transition(project, "deposit")
        view = queryMultiAdapter((self.campaign, self.request), name="faceted-explorer")
        self.assertIn("My first project", view())
        self.assertNotIn("@@images/project_image/project_faceted", view())
        self.assertNotIn('<span class="theme"', view())
        self.assertNotIn('id="btn-create-projects"', view())
        img = NamedBlobImage(**make_named_image())
        project.project_image = img
        self.assertIn("@@images/project_image/project_faceted", view())
        project.project_theme = ["PROP"]
        self.assertIn('<span class="theme"', view())

        self.campaign.project_submission = True
        view = queryMultiAdapter((self.campaign, self.request), name="faceted-explorer")
        self.assertIn('id="btn-create-projects"', view())

        api.portal.set_registry_record(
            "ideabox.policy.browser.controlpanel.IIdeaBoxSettingsSchema.ts_project_submission_path",
            "https://e-guichet.imio.be/",
        )
        self.assertIn("https://e-guichet.imio.be/", view())



# -*- coding: utf-8 -*-

from ideabox.policy.testing import IdeaboxTestCase
from ideabox.policy.testing import IDEABOX_POLICY_INTEGRATION_TESTING
from ideabox.policy.tests.utils import make_named_image
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from zope.component import queryMultiAdapter


class TestRestServices(IdeaboxTestCase):

    layer = IDEABOX_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_theme_project_get(self):
        pass

    def test_district_project_get(self):
        pass



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.User import Super as BaseUnrestrictedUser
from collective.taxonomy.interfaces import ITaxonomy
from datetime import datetime
from eea.facetednavigation.layout.layout import FacetedLayout
from ideabox.policy import _
from plone import api
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id."""

    def getId(self):
        """Return the ID of the user."""
        return self.getUserName()


def execute_under_admin(portal, function, *args, **kwargs):
    """Execude code under admin privileges"""
    sm = getSecurityManager()
    try:
        try:
            tmp_user = UnrestrictedUser("admin", "", [""], "")
            # Wrap the user in the acquisition context of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)
            # Call the function
            return function(*args, **kwargs)
        except:
            # If special exception handlers are needed, run them here
            raise
    finally:
        # Restore the old security manager
        setSecurityManager(sm)


def review_state(context):
    return api.content.get_state(obj=context)


def can_view_rating(context):
    _rating_states = ("vote", "result_analysis", "rejected")
    return review_state(context) in _rating_states


def now():
    return datetime.now()


def activate_faceted_navigation(context, config_path=""):
    subtyper = context.restrictedTraverse("@@faceted_subtyper")
    if subtyper.is_faceted:
        return
    subtyper.enable()
    with open(config_path, "rb") as config:
        context.unrestrictedTraverse("@@faceted_exportimport").import_xml(
            import_file=config
        )


def set_faceted_view(context, view_name):
    layout = FacetedLayout(context)
    layout.update_layout(layout=view_name)


def localized_month(value, request):
    keys = {
        "January": _("January"),
        "February": _("February"),
        "March": _("March"),
        "April": _("April"),
        "May": _("May"),
        "June": _("June"),
        "July": _("July"),
        "August": _("August"),
        "September": _("September"),
        "October": _("October"),
        "November": _("November"),
        "December": _("December"),
        "Jan": _("Jan"),
        "Feb": _("Feb"),
        "Mar": _("Mar"),
        "Apr": _("Apr"),
        "Jun": _("Jun"),
        "Jul": _("Jul"),
        "Aug": _("Aug"),
        "Sep": _("Sep"),
        "Oct": _("Oct"),
        "Nov": _("Nov"),
        "Dec": _("Dec"),
    }
    for k, v in keys.items():
        if k in value:
            value = value.replace(k, translate(v, context=request))
            return value
    return value


def disable_portlets(context, disabled=("plone.leftcolumn", "plone.rightcolumn")):
    for manager_name in disabled:
        manager = getUtility(IPortletManager, name=manager_name)
        blacklist = getMultiAdapter((context, manager), ILocalPortletAssignmentManager)
        blacklist.setBlacklistStatus(CONTEXT_CATEGORY, True)


def get_district():
    sm = getSiteManager()
    return sm.queryUtility(ITaxonomy, name="collective.taxonomy.district")


def get_theme():
    sm = getSiteManager()
    return sm.queryUtility(ITaxonomy, name="collective.taxonomy.theme")


def get_vocabulary(voc_name):
    factory = getUtility(IVocabularyFactory, voc_name)
    vocabulary = factory(api.portal.get())
    return vocabulary
